/*
@file    FT800_commands.h
@brief   Contains FT800 Function Prototypes
@version 2.4
@date    2017-02-16
@author  Rudolph Riedel

@section History

2.2
- changes to this header

2.3

- changed type of result-value for ft800_init from void to uint8_t
- added prototype for ft800_cmd_romfont()
- added millis option to ft800_cmd_clock()
- switched to MISRA compliant comment-style

2.4
- added prototype for ft800_memWrite_flash_buffer()

*/

#ifndef FT800_COMMANDS_H_
#define FT800_COMMANDS_H_

extern uint16_t cmdOffset;

void ft800_cmdWrite(uint8_t data);

uint8_t ft800_memRead8(uint32_t ftAddress);
uint16_t ft800_memRead16(uint32_t ftAddress);
uint32_t ft800_memRead32(uint32_t ftAddress);
void ft800_memWrite8(uint32_t ftAddress, uint8_t ftData8);
void ft800_memWrite16(uint32_t ftAddress, uint16_t ftData16);
void ft800_memWrite32(uint32_t ftAddress, uint32_t ftData32);
void ft800_memWrite_flash_buffer(uint32_t ftAddress, const uint8_t *data, uint16_t len);
uint8_t ft800_busy(void);
void ft800_cmd_dl(uint32_t command);

void ft800_get_cmdoffset(void);
uint32_t ft800_get_touch_tag(void);
void ft800_cmd_execute(void);


/* commands to draw graphics objects: */
void ft800_cmd_text(int16_t x0, int16_t y0, int16_t font, uint16_t options, const char* text);
void ft800_cmd_button(int16_t x0, int16_t y0, int16_t w0, int16_t h0, int16_t font, uint16_t options, const char* text);
void ft800_cmd_clock(int16_t x0, int16_t y0, int16_t r0, uint16_t options, uint16_t hours, uint16_t minutes, uint16_t seconds, uint16_t millisecs);
void ft800_cmd_bgcolor(uint32_t color);
void ft800_cmd_fgcolor(uint32_t color);
void ft800_cmd_gradcolor(uint32_t color);
void ft800_cmd_gauge(int16_t x0, int16_t y0, int16_t r0, uint16_t options, uint16_t major, uint16_t minor, uint16_t val, uint16_t range);
void ft800_cmd_gradient(int16_t x0, int16_t y0, uint32_t rgb0, int16_t x1, int16_t y1, uint32_t rgb1);
void ft800_cmd_keys(int16_t x0, int16_t y0, int16_t w0, int16_t h0, int16_t font, uint16_t options, const char* text);
void ft800_cmd_progress(int16_t x0, int16_t y0, int16_t w0, int16_t h0, uint16_t options, uint16_t val, uint16_t range);
void ft800_cmd_scrollbar(int16_t x0, int16_t y0, int16_t w0, int16_t h0, uint16_t options, uint16_t val, uint16_t size, uint16_t range);
void ft800_cmd_slider(int16_t x1, int16_t y1, int16_t w1, int16_t h1, uint16_t options, uint16_t val, uint16_t range);
void ft800_cmd_dial(int16_t x0, int16_t y0, int16_t r0, uint16_t options, uint16_t val);
void ft800_cmd_toggle(int16_t x0, int16_t y0, int16_t w0, int16_t font, uint16_t options, uint16_t state, const char* text);
void ft800_cmd_number(int16_t x0, int16_t y0, int16_t font, uint16_t options, int32_t number);

#ifdef FT_81X_ENABLE
void ft800_cmd_setbase(uint32_t base);
void ft800_cmd_setbitmap(uint32_t addr, uint16_t fmt, uint16_t width, uint16_t height);
#endif

/* commands to operate on memory: */
void ft800_cmd_memzero(uint32_t ptr, uint32_t num);
void ft800_cmd_memset(uint32_t ptr, uint8_t value, uint32_t num);
/*(void ft800_cmd_memwrite(uint32_t dest, uint32_t num, const uint8_t *data); */
void ft800_cmd_memcpy(uint32_t dest, uint32_t src, uint32_t num);
void ft800_cmd_append(uint32_t ptr, uint32_t num);


/* commands for loading image data into FT800 memory: */
void ft800_cmd_inflate(uint32_t ptr, const uint8_t *data, uint16_t len);
void ft800_cmd_loadimage(uint32_t ptr, uint32_t options, const uint8_t *data, uint16_t len);
#ifdef FT_81X_ENABLE
void ft800_cmd_mediafifo(uint32_t ptr, uint32_t size);
#endif

/* commands for setting the bitmap transform matrix: */
void ft800_cmd_loadidentity(void);
void ft800_cmd_translate(int32_t tx, int32_t ty);
void ft800_cmd_scale(int32_t sx, int32_t sy);
void ft800_cmd_rotate(int32_t ang);
void ft800_cmd_setmatrix(void);
void ft800_cmd_getmatrix(int32_t a, int32_t b, int32_t c, int32_t d, int32_t e, int32_t f);


/* other commands: */
void ft800_cmd_calibrate(void);
void ft800_cmd_coldstart(void);
void ft800_cmd_interrupt(uint32_t ms);
void ft800_cmd_logo(void);
void ft800_cmd_screensaver(void);
#ifdef FT_81X_ENABLE
void ft800_cmd_setrotate(uint32_t r);
void ft800_cmd_romfont(uint32_t font, uint32_t romslot);
#endif
void ft800_cmd_sketch(int16_t x0, int16_t y0, uint16_t w0, uint16_t h0, uint32_t ptr, uint16_t format);
void ft800_cmd_snapshot(uint32_t ptr);
void ft800_cmd_spinner(int16_t x0, int16_t y0, uint16_t style, uint16_t scale);
void ft800_cmd_stop(void);
void ft800_cmd_track(int16_t x0, int16_t y0, int16_t w0, int16_t h0, int16_t tag);


/* meta-commands, sequences of several display-list entries condensed into simpler to use functions at the price of some overhead */
void ft800_cmd_point(int16_t x0, int16_t y0, uint16_t size);
void ft800_cmd_line(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t w0);
void ft800_cmd_rect(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t corner);


/* startup FT800: */
uint8_t ft800_init(void);

#endif /* FT800_COMMANDS_H_ */